$title	Repeal of the Corn Laws -- A Static General Equilibrium Model

*	tm0		Base year tariff rate on grain
*	epsilon		Supply elasticity of cotton imports 
*	eta		Demand elasticity of textile exports
*	etaL		Elasticity of transformation in land

$if not set tm0     $set tm0 0.54
$if not set epsilon $set epsilon 10
$if not set eta     $set eta  10
$if not set etaL    $set etaL 10

set s	Sectors /
	grn	Grains
	agr	Pastoral agriculture
	min	Mining
	foo	Food etc.
	mtl	Metal
	soa	Soap
	tex	Textiles
	mgd	Metal Goods
	brk	Bricks
	oth	Other mfg
	con	Construction
	gas	Gas
	trn	Transport
	dis	Distribution
	ser	Domestic Service
	ots	Other service
	pub	Public admin
	hou	Housing /;


parameter	iotable(*,*)	Input-output table;
$if not exist iotable.gdx $call 'gdxxrw i=uk1841rev.xls o=iotable.gdx par=iotable rng=iotable!c3..x25 rdim=1 cdim=1 par=kvs rng="shares!a18" rdim=1 cdim=0'
$gdxin iotable.gdx
$load iotable

alias (s,g);

parameter	id0(g,s)	Intermediate demand,
		m0(s)		Imports,
		t0(s)		Indirect taxes,
		va0(s)		Value-added,
		e0(g)		Exports,
		c0(g)		Consumption demand,
		t0(g)		Indirect taxes,
		i0(g)		Investment demand;

id0(g,s) = iotable(g,s);
m0(s) = iotable("imp",s);
t0(s) = iotable("tax",s);
va0(s) = iotable("va",s);
c0(g) = iotable(g,"cons");
i0(g) = iotable(g,"inv");
e0(g) = iotable(g,"exp");


parameter	bydata	Base year statistics on goods;
bydata(g,"id0") = sum(s, id0(g,s));
bydata(g,"m0") = m0(g);
bydata(g,"e0") = e0(g);
bydata(g,"c0") = c0(g);
bydata(g,"i0") = i0(g);
option bydata:1;
display bydata;

parameter	y0(s)	Sectoral output;
y0(g) = sum(s, id0(g,s)) + c0(g) + i0(g) + e0(g) - m0(g);

parameter	profit(s,*)	Cross check on profit;
profit(s,"value") = y0(s) - sum(g, id0(g,s)) - t0(s) - va0(s);
profit(s,"percent") = 100 * profit(s,"value")/y0(s);
display profit;

parameter	supdata	Base year statistics on sectors;
supdata(s,"id0") = sum(g,id0(g,s));
supdata(s,"id0(s,s)") = id0(s,s);
supdata(s,"y0") = y0(s);
supdata(s,"t0") = t0(s);
supdata(s,"va0") = va0(s);
option supdata:1;
display supdata;


set f Primary factors /
	K	Capital,
	N	Land
	L	Labor /;

set	mf(f)	Mobile factors /K,L/
	sf(f)	Sluggish factors /N/;


table fshare(s,f)	Factor shares of value-added

*	N.B.  These values are more-or-less randomly assigned.

	K	N	L	
grn	0.3	0.4	0.3
agr	0.2	0.5	0.3
min	0.35		0.65
foo	0.35		0.65
mtl	0.35		0.65
soa	0.35		0.65
tex	0.35		0.65
mgd	0.35		0.65
brk	0.35		0.65
oth	0.35		0.65
con	0.5		0.5
gas	0.5		0.5
trn	0.5		0.5
dis	0.5		0.5
ser	0.5		0.5
ots	0.5		0.5
pub	0.5		0.5
hou	0.5		0.5;

set	as	Aggregate sectors (for capital value share information) /
	agr	Agriculture
	min	Mining
	mfr	Manufacturing
	con	Construction
	gas	Natural gas and water,
	trn	Transportation
	dis	Distribution
	pub	Public and private services
	hou	Housing /;

set	smap(s,as) /
	grn.agr	Grains
	agr.agr	Pastoral agriculture
	min.min	Mining
	foo.mfr	Food etc.
	mtl.mfr	Metal
	soa.mfr	Soap
	tex.mfr	Textiles
	mgd.mfr	Metal Goods
	brk.mfr	Bricks
	oth.mfr	Other mfg
	con.con	Construction
	gas.gas	Gas
	trn.trn	Transport
	dis.dis	Distribution
	ser.pub	Domestic Service
	ots.pub	Other service
	pub.pub	Public admin
	hou.hou	Housing /;


parameter	kvs(as)		Capital value shares;
$load kvs

$if not set kvs	    $set kvs yes
if (%kvs%,
	loop(smap(s,as),
		fshare(s,"k") = (fshare(s,"k")+fshare(s,"l")) * kvs(as);
		fshare(s,"L") = (1-kvs(as))*fshare(s,"k")/kvs(as);
	);
);

parameter	ty(s)		Indirect tax rate
		tm(g)		Import tariff rate
		fd0(f,s)	Factor demand
		pm0(g)		Reference price of imports
		fe0(f)		Factor endowments,
		bopdef		Balance of payments deficit,
		eta(g)		Export demand elasticity,
		etaL		Elasticity of transformation for land /%etaL%/
		epsilon(g)	Import supply elasticity
		im0(g)		Imported intermediate inputs
		sigma(g)	Armington import demand elasticity,
		d0(g)		Domestic demand
		a0(g)		Aggregate demand;

fd0(f,s) = va0(s) * fshare(s,f);
fe0(f) = sum(s, fd0(f,s));
display fe0;

ty(s) = t0(s)/y0(s);

*	Import tariff on grain:

tm("grn") = %tm0%;

*	The remaining 12.1 in imports used by the food
*	sector is mainly sugar, tea, coffee, spirits, etc. and these imports
*	were also taxed - I'd put the existing average duty at 100% in this
*	category, although this tariff will not be removed in our simulation

tm("foo") = 1.00;

*	Williamson also has a 5.3 percent tariff on pastoral agriculture.
*	That should be imposed and removed along with the tariff on grains.

tm("agr") = 0.053;

pm0(g) = 1 + tm(g);
bopdef = sum(g, m0(g)/pm0(g)) - sum(s, e0(s));
eta(g) = 10;
eta("tex") = %eta%;
epsilon(g) = 10;
epsilon("tex") = %epsilon%;
sigma(g) = 10;
im0(g) = 0;

*	The only other adjustment that might be important and that you might
*	want to build in is the 22.2 in imports used by the textile sector.
*	Of this, 12 is raw cotton.  The rest are intermediates that compete
*	with pastoral production (wool, hides, flax, etc.).  

im0("tex") = 12;
id0("tex","tex") = id0("tex","tex") - im0("tex");
d0(g) = y0(g) - e0(g);
a0(g) = d0(g) + m0(g) - im0(g);

$ontext
$model:uk1841

$sectors:
	y(s)		! Sectoral output
	a(s)		! Armington demand
	fs(sf)		! Factor supply (CET)
	con		! Consumption

$commodities:
	pc
	p(g)		! Commodity prices
	pd(g)		! Domestic output price
	pm(g)$m0(g)	! Imported goods price 
	w(f)		! Factor prices
	pf(sf,s)$fd0(sf,s)	! Sluggish factor prices
	pfx			! Price of foreign exchange

$consumers:
	ra	! Representative agent

$auxiliary:
	e(g)		! Export demand
	m(g)$m0(g)	! Import supply
	ve		! Value of aggregate exports

$prod:y(s)  s:0  va:1
	o:pd(s)	q:y0(s)			a:ra	t:ty(s)
	i:p(g)	q:id0(g,s)
	i:pm(s)	q:(im0(s)/pm0(s))	a:ra t:tm(s) p:pm0(s)
	i:w(mf)		q:fd0(mf,s)		va:
	i:pf(sf,s)	q:fd0(sf,s)		va:

$prod:fs(sf)  t:etaL
	o:pf(sf,s)	q:fd0(sf,s)
	i:w(sf)		q:fe0(sf)

$report:
	v:fdm(mf,s)$fd0(mf,s)	i:w(mf)		prod:y(s)
	v:fds(sf,s)$fd0(sf,s)	i:pf(sf,s)	prod:y(s)

$prod:a(g)  s:sigma(g)
	o:p(g)	q:a0(g)
	i:pd(g)	q:d0(g)
	i:pm(g)	q:((m0(g)-im0(g))/pm0(g))  a:ra t:tm(g) p:pm0(g)

$demand:ra  s:1
	e:w(f)	q:fe0(f)
	e:pd(g)	q:(-1)		r:e(g)
	e:pm(g)$m0(g)	q:1	r:m(g)
	e:pfx	q:1		r:ve
	e:p(g)	q:(-i0(g))
	e:pfx	q:bopdef
	d:pc

$prod:con
	o:pc	q:(sum(g,c0(g)))
	i:p(g)	q:c0(g)

$constraint:ve
	pfx * ve =e= sum(g, pd(g)*e(g) - pm(g)*m(g));

$constraint:m(g)$m0(g)
	m(g) =e= m0(g)/pm0(g) * (pm(g)/pfx)**epsilon(g);

$constraint:e(g)
	e(g) =e= e0(g) * (pd(g)/pfx)**(-eta(g));

$offtext
$sysinclude mpsgeset uk1841

e.l(g) = e0(g);
e.fx(g)$(e0(g)=0) = 0;
m.l(g) = m0(g)/pm0(g);
m.fx(g)$(m0(g)=0) = 0;
pm.fx(g)$(m0(g)=0) = 0;
ve.lo = -inf;
ve.l = sum(g, e.l(g) - m.l(g));

uk1841.iterlim = 0;
$include UK1841.GEN
solve uk1841 using mcp;


*	Repeal the import tariff on agricultural products:

tm("grn") = 0;
tm("agr") = 0;
uk1841.iterlim = 10000;
$include UK1841.GEN
solve uk1841 using mcp;

*	Report impacts:

parameter	quant(*,*)	Quantitative impact by sector (level)
		qpct(*,*)	Quantitative impact by sector (% change)
		gprice(*,*)	Goods price impacts (% change)
		fprice(*,*)	Factor price impacts (% change)
		hev		Welfare impact (%);

qpct(g,"m")$m0(g) = (m.l(g)/(m0(g)/pm0(g)) - 1);
qpct(s,"y") = 100 * (y.l(s) - 1);
qpct(g,"e")$e0(g) = 100 * (e.l(g)/e0(g)-1);
qpct(s,"N")$fd0("N",s) = 100 * (fds.l("N",s)/fd0("N",s)-1);
qpct(s,"L")$fd0("L",s) = 100 * (fdm.l("L",s)/fd0("L",s)-1);
qpct(s,"K")$fd0("K",s) = 100 * (fdm.l("K",s)/fd0("K",s)-1);

quant(g,"m")$m0(g) = m.l(g) - (m0(g)/pm0(g));
quant(s,"y") = y0(s) * (y.l(s) - 1);
quant(g,"e")$e0(g) = e.l(g) - e0(g);
quant(s,"N")$fd0("N",s) = fds.l("N",s) - fd0("N",s);
quant(s,"L")$fd0("L",s) = fdm.l("L",s) - fd0("L",s);
quant(s,"K")$fd0("K",s) = fdm.l("K",s) - fd0("K",s);

gprice(s,"p/pc") = 100 * (p.l(s)/pc.l - 1);
gprice(s,"pd/pc") = 100 * (pd.l(s)/pc.l - 1);
gprice(s,"p/pfx") = 100 * (p.l(s)/pfx.l - 1);
gprice(s,"pd/pfx") = 100 * (pd.l(s)/pfx.l - 1);
loop(s$(m0(s)<>0),
	gprice(s,"pm/pd")$pd.l(s) = 100 * (pm.l(s)/pd.l(s) - 1);
);

fprice(f,"pf/pfx") = 100 * (w.l(f)/pfx.l-1);
fprice(f,"pf/pd") = 100 * (w.l(f)*sum(s,y0(s))/sum(s, pd.l(s)*y0(s))-1);
fprice(f,"pf/pc") = 100 * (w.l(f)*sum(g,c0(g))/sum(g, p.l(g)*c0(g))-1);

hev = 100 * (con.l - 1);

option decimals=1;
display hev, qpct, quant, gprice, fprice;
